module Locraft
  IOS = 'iOS'.freeze
  ANDROID = 'android'.freeze

  SWIFT = 'swift'.freeze
  OBJC = 'objc'.freeze
  JAVA = 'java'.freeze

  STRINGS_EXT = '.strings'.freeze
  XML_EXT = '.xml'.freeze

  class Config
    attr_accessor :platform
    attr_accessor :gdoc_file
    attr_accessor :strings_destination
    attr_accessor :strings_basename
    attr_accessor :langs
    attr_accessor :default_lang
    attr_accessor :gdoc_sheet
    attr_accessor :gdoc_keys_column
    attr_accessor :gdoc_comments_column
    attr_accessor :keys_map

    # fill automatically in load_from method
    attr_accessor :from_file

    def initialize
      self.default_lang = 'English'
      self.gdoc_sheet = 0
      self.gdoc_keys_column = 'Keys'
      self.gdoc_comments_column = 'Comments'
      self.keys_map = {}
      self.strings_basename = 'Localizable'
      self.langs = {
        'English' => 'en',
        'Russian' => 'ru'
      }
      yield self if block_given?
    end

    def self.load_from(file)
      return unless file || File.file?(file)
      begin
        config = eval(File.read(file), binding, 'config loading problem')
        config.from_file = file
        return config if config.is_a? Config

        warn "[#{file}] isn't a Locraft::Config, but #{config.class}."
      rescue SyntaxError, StandardError => e
        warn "Invalid config in [#{file}]: #{e}"
      end
    end

    def from_folder
      File.dirname(from_file)
    end

    def relative_strings_destination
      File.expand_path('../' + strings_destination, from_file)
    end

    def strings_file
      raise NotImplementedError
    end

    def strings_generators
      raise NotImplementedError
    end
  end

  class IOSConfig < Config
    attr_accessor :info_plist_destination
    attr_accessor :info_plist_basename
    attr_accessor :macro_destination
    attr_accessor :dev_lang
    attr_accessor :dev_prefix
    attr_accessor :keys_map
    attr_accessor :macro_file

    def initialize
      super()
      self.platform = IOS
      self.dev_lang = OBJC
      self.dev_prefix = 'XYZ'
      self.macro_file = 'LocalizedConstants'
      self.info_plist_basename = 'InfoPlist'
      yield self if block_given?
    end

    def relative_plist_destination
      return nil if info_plist_destination.nil?
      File.expand_path('../' + info_plist_destination, from_file)
    end

    def relative_macro_destination
      return nil if macro_destination.nil?
      File.expand_path('../' + macro_destination, from_file)
    end

    def strings_file
      "#{strings_basename}#{STRINGS_EXT}"
    end

    def info_plist_file
      "#{info_plist_basename}#{STRINGS_EXT}"
    end

    def strings_generators
      [StringsGenerator.new(self), info_plist_generator, constants_generator].select(&:valid?)
    end

    def info_plist_generator
      InfoPlistGenerator.new(self)
    end

    def constants_generator
      if dev_lang == SWIFT
        SwiftConstantsGenerator.new(self)
      elsif dev_lang == OBJC
        ObjcConstantsGenerator.new(self)
      end
    end
  end

  class AndroidConfig < Config
    def initialize
      super()
      self.platform = ANDROID
      self.strings_basename = 'strings'
      yield self if block_given?
    end

    def strings_file
      "#{strings_basename}#{XML_EXT}"
    end

    def strings_generators
      [XMLStringsGenerator.new(self)]
    end
  end
end
